@@ -226,6 +226,9 @@ def InlineMux2(obj:Logic):
 def InlineAdd(obj:Logic):
     return "assign {} = {} + {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
 
+def InlineDiv(obj:Logic):
+    return "assign {} = {} / {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
+
 def InlineSub(obj:Logic):
     return "assign {} = {} - {};\n".format(getParentWireName(obj, obj.r), getParentWireName(obj, obj.a) , getParentWireName(obj, obj.b))
 
@@ -375,6 +378,7 @@ class VerilogGenerator:
         self.inlinablePrimitives[BitsMSBF] = InlineBitsMSBF
         self.inlinablePrimitives[ConcatenateMSBF] = InlineConcatenateMSBF
         self.inlinablePrimitives[Constant] = InlineConstant
+        self.inlinablePrimitives[Div] = InlineDiv
         self.inlinablePrimitives[Equal] = InlineEqual
         self.inlinablePrimitives[EqualConstant] = InlineEqualConstant
         self.inlinablePrimitives[Nand2] = InlineNand2
@@ -583,11 +587,22 @@ class VerilogGenerator:
         wireName = getWireNames(child.parent)
         
         for inp in child.inPorts:
-            
+            if (inp.wire is None):
+                raise Exception('Input port {} from {} not connected to any wire'.format(inp.name, child.getFullPath()));
+                
+            if (not(inp.wire in wireName)):
+                raise Exception('Input port wire {} not part of the wires of the parent {}'.format(inp.wire.getFullPath(), child.parent.getFullPath()))
+                
             str += link +  "." + getPortName(inp) + "("+wireName[inp.wire]+")"
             link = ","
 
         for outp in child.outPorts:
+            if (outp.wire is None):
+                raise Exception('Output port {} from {} not connected to any wire'.format(outp.name, child.getFullPath()));
+
+            if (not(outp.wire in wireName)):
+                raise Exception('Output port wire {} not part of the wires of the parent {}'.format(outp.wire.getFullPath(), child.parent.getFullPath()))
+
             str += link + "." + getPortName(outp) + "("+wireName[outp.wire]+")"
  
             link = ","
@@ -627,8 +642,8 @@ class VerilogGenerator:
         
         str += "// local declarations\n"
         str += tr.getExtraDeclarations() + "\n";
-        str += "// sequential process\n"
-        str += "always @(posedge {})\n".format(clkname)
+        str += "// combinational process\n"
+        str += "always @(*)\n".format(clkname)
         str += "begin\n"
         str += transpiled + "\n"
         str += "end\n"
@@ -691,6 +706,7 @@ class Python2VerilogTranspiler:
         
         module = ReplaceWireGets().visit(module)
         module = ReplaceWirePrepare().visit(module)
+        module = ReplaceWirePut().visit(module)
         
         func = module.body[0].body
         
@@ -731,6 +747,8 @@ class Python2VerilogTranspiler:
             return self.transpileCompare(line)
         if (type(line) == ast.Assign):
             return self.transpileAssign(line)
+        if (type(line) == ast.BinOp):
+            return self.transpileBinOp(line)
         if (type(line) == ast.Attribute):
             return self.transpileAttribute(line)
         if (isinstance(line, ast.Name)):
@@ -764,6 +782,19 @@ class Python2VerilogTranspiler:
     def transpileExpr(self, line:ast.Expr):
         return self.transpileUnknown(line.value)
         
+    def transpileBinOp(self, line:ast.BinOp):
+        return "(" + self.transpileUnknown(line.left) + self.transpileOp(line.op) + self.transpileUnknown(line.right) + ")";
+
+    def transpileOp(self, op):
+        if (isinstance(op, ast.BitAnd)):
+            return '&'
+        elif (isinstance(op, ast.BitOr)):
+            return '|'
+        elif (isinstance(op, ast.BitXor)):
+            return '^'
+        else:
+            raise Exception('Op not supported: {}'.format(op))
+        
     def transpileAssign(self, line:ast.Assign):
         targets = line.targets 
         if (len(targets) > 1):
@@ -831,4 +862,15 @@ class ReplaceWirePrepare(ast.NodeTransformer):
             return ast.Assign([node.func.value], node.args[0])
         
         
+        return node
+    
+class ReplaceWirePut(ast.NodeTransformer):
+    def visit_Call(self, node):
+        attr = getAstName(node.func)
+        
+        if (attr == 'put'):
+            #print('REPLACE WIRE PUTS FUNC:', attr , node.func.value.attr, node.args)
+            return ast.Assign([node.func.value], node.args[0])
+        
+        
         return node
\ No newline at end of file
